progression_list = [
    {
        'degrees_repr': ['I', 'V', 'vi', 'IV'],
        'degress_halftones': [],
        'scale': 'major',
        'examples': {
            'The Beatles': [
                'Let it Be'
            ]
        }
    },

    {
        'degrees_repr': ['i', 'VII', 'VI'],
        'degress_halftones': [],
        'scale': 'minor',
        'examples': {
            'Led Zeppelin': [
                'Stairway to Heaven'
            ]
        }
    }
]
